<?php

/**
 * Plugin Name: Extraction of products and categories for Mazzaneh - Official
 * Description: افزونه ای برای استخراج تمامی محصولات و دسته بندی های ووکامرس
 * Version: 1.2.0
 * Author: Mazzaneh
 * Author URI: https://mazzaneh.ir/
 * License: MIT
 * License URI: https://opensource.org/licenses/MIT
 * Text Domain: woocommerce-category-product-extractor
 */

// Exit if accessed directly
if (!defined('ABSPATH')) {
  exit;
}

// Check if WooCommerce is active
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
  class Mazzaneh_Woocommerce_Categories_Products_Extractor extends WP_REST_Controller
  {
    private $mazzaneh_wcpe_version = "1.2.0";

    public function __construct()
    {
      add_action('rest_api_init', array($this, 'mazzaneh_wcpe_register_categories_route'));
      add_action('rest_api_init', array($this, 'mazzaneh_wcpe_register_products_route'));
    }

    /**
     * Register route: https://domain.com/mazzaneh_wcpe/v1/categories
     */
    public function mazzaneh_wcpe_register_categories_route()
    {
      $version = '1';
      $namespace = 'mazzaneh_wcpe/v' . $version;
      $base = 'categories';
      register_rest_route(
        $namespace,
        '/' . $base,
        array(
          array(
            'methods' => 'POST',
            'callback' => array(
              $this,
              'mazzaneh_wcpe_get_categories'
            ),
            'permission_callback' => '__return_true',
            'args' => array()
          )
        )
      );
    }

    /**
     * Register route: https://domain.com/mazzaneh_wcpe/v1/products
     */
    public function mazzaneh_wcpe_register_products_route()
    {
      $version = '1';
      $namespace = 'mazzaneh_wcpe/v' . $version;
      $base = 'products';
      register_rest_route(
        $namespace,
        '/' . $base,
        array(
          array(
            'methods' => 'POST',
            'callback' => array(
              $this,
              'mazzaneh_wcpe_get_products'
            ),
            'permission_callback' => '__return_true',
            'args' => array()
          )
        )
      );
    }

    /**
     * Get all of categories
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function mazzaneh_wcpe_get_categories($request)
    {
      // Get Parameters
      $limit = intval($request->get_param('limit'));
      $page = intval($request->get_param('page'));

      // Check request is valid and update
      $response = $this->mazzaneh_wcpe_check_request($request);
      if (!is_array($response)) {
        $data['Response'] = '';
        $data['Error'] = $response;
        $response_code = 500;
      } else {
        $response_body = $response['body'];
        $response = json_decode($response_body, true);

        if ($response['success'] === TRUE && $response['message'] === 'the token is valid') {
          $data = $this->mazzaneh_wcpe_get_all_categories($limit, $page);
          $response_code = 200;
        } else {
          $data['Response'] = $response_body;
          $data['Error'] = $response['error'];
          $response_code = 401;
        }
      }
      $data['Version'] = $this->mazzaneh_wcpe_version;
      return new WP_REST_Response($data, $response_code);
    }

    /**
     * Get all of products
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function mazzaneh_wcpe_get_products($request)
    {
      // Get Parameters
      $limit = intval($request->get_param('limit'));
      $page = intval($request->get_param('page'));

      // Check request is valid and update
      $response = $this->mazzaneh_wcpe_check_request($request);
      if (!is_array($response)) {
        $data['Response'] = '';
        $data['Error'] = $response;
        $response_code = 500;
      } else {
        $response_body = $response['body'];
        $response = json_decode($response_body, true);

        if ($response['success'] === TRUE && $response['message'] === 'the token is valid') {
          $data = $this->mazzaneh_wcpe_get_all_products($limit, $page);
          $response_code = 200;
        } else {
          $data['Response'] = $response_body;
          $data['Error'] = $response['error'];
          $response_code = 401;
        }
      }
      $data['Version'] = $this->mazzaneh_wcpe_version;
      return new WP_REST_Response($data, $response_code);
    }

    /**
     * Check update and validate the request
     * @param request
     * @return wp_safe_remote_post
     */
    public function mazzaneh_wcpe_check_request($request)
    {
      // Get shop domain
      $site_url = wp_parse_url(get_site_url());
      $shop_domain = str_replace('www.', '', $site_url['host']);

      // Mazzaneh verify token url
      $endpoint_url = 'https://adminapi.mazzaneh.ir/api/gallery/validate_token/';

      // Get Parameters
      $token = sanitize_text_field($request->get_param('token'));

      // Get Headers
      $header = $request->get_header('X-Authorization');
      if (empty($header)) {
        $header = $request->get_header('Authorization');
      }

      // Verify token
      $response = wp_safe_remote_post(
        $endpoint_url,
        array(
          'method' => 'POST',
          'timeout' => 5,
          'redirection' => 0,
          'httpversion' => '1.0',
          'blocking' => true,
          'headers' => array(
            'AUTHORIZATION' => $header,
          ),
          'body' => array(
            'token' => $token,
            'shop_domain' => $shop_domain,
            'version' => $this->mazzaneh_wcpe_version,
          ),
          'cookies' => array()
        )
      );

      return $response;
    }

    /**
     * Get all categories
     *
     * @param int $limit Number of categories to retrieve per page.
     * @param int $page Page number for pagination.
     * @return array
     */
    private function mazzaneh_wcpe_get_all_categories($limit, $page)
    {
      $args = array(
        'taxonomy' => 'product_cat', // WooCommerce product category taxonomy
        'number' => $limit,
        'offset' => ($page - 1) * $limit, // Offset calculation for pagination
        'hide_empty' => false, // Show empty categories as well
        'orderby' => 'id',
        'order' => 'DESC',
      );

      // Get WooCommerce product categories
      $categories = get_terms($args);

      // Get total categories count
      $total_categories = intval(wp_count_terms('product_cat'));

      // Calculate total pages
      $max_pages = ceil($total_categories / $limit);

      // Prepare the response data
      $data = array(
        'count' => $total_categories,
        'max_pages' => $max_pages,
        'categories' => array(),
      );

      // Retrieve and send data in json
      foreach ($categories as $category) {
        $temp_category = $this->mazzaneh_wcpe_get_category_values($category);
        $data['categories'][] = $temp_category;
      }

      return $data;
    }

    /**
     * Get all products
     *
     * @param int $limit Number of categories to retrieve per page.
     * @param int $page Page number for pagination.
     * @return array
     */
    private function mazzaneh_wcpe_get_all_products($limit, $page)
    {
      $args = array(
        'posts_per_page' => $limit,
        'offset' => ($page - 1) * $limit, // Offset calculation for pagination
        'post_status' => 'publish',
        'orderby' => 'ID',
        'order' => 'DESC',
        'post_type' => 'product',
      );

      // Make query
      $query = new WP_Query($args);

      // Get products
      $products = $query->get_posts();

      // Get total products count
      $total_products = $query->found_posts;

      // Calculate total pages
      $max_pages = $query->max_num_pages;

      // Prepare the response data
      $data = array(
        'count' => $total_products,
        'max_pages' => $max_pages,
        'products' => array(),
      );

      // Retrieve and send data in json
      foreach ($products as $product) {
        $product = wc_get_product($product->ID);
        $temp_product = $this->mazzaneh_wcpe_get_product_values($product);
        $data['products'][] = $temp_product;
      }

      return $data;
    }

    /**
     * Get single category values
     */
    public function mazzaneh_wcpe_get_category_values($category)
    {
      $temp_category = new \stdClass();

      $temp_category->id = $category->term_id;
      $temp_category->name = $category->name;

      return $temp_category;
    }

    /**
     * Get single product values
     */
    public function mazzaneh_wcpe_get_product_values($product)
    {
      $temp_product = new \stdClass();

      $temp_product->id = $product->get_id();
      $temp_product->name = $product->get_name();
      $temp_product->url = get_permalink($product->get_id());
      $temp_product->images = [];
      $attachment_ids = $product->get_gallery_image_ids();
      foreach ($attachment_ids as $attachment_id) {
        $t_link = wp_get_attachment_image_src($attachment_id, 'full');
        if ($t_link) {
          array_push($temp_product->images, $t_link[0]);
        }
      }
      $t_image = wp_get_attachment_image_src($product->get_image_id(), 'full');
      if ($t_image) {
        if (!in_array($t_image[0], $temp_product->images)) {
          array_push($temp_product->images, $t_image[0]);
        }
      }
      $temp_product->price = $product->get_price();
      $temp_product->old_price = $product->get_regular_price();
      $temp_product->short_desc = $product->get_short_description();
      $temp_product->description = $product->get_description();
      $temp_product->attributes = array();
      $cat_ids = $product->get_category_ids();
      $temp_product->category = get_term_by('id', end($cat_ids), 'product_cat', 'ARRAY_A')['name'];
      $temp_product->availability = $product->get_stock_status();

      if ($product->is_type('variable')) {
        // Set prices to 0 then calculate them
        $temp_product->price = 0;
        $temp_product->old_price = 0;

        // Find price for default attributes. If can't find return max price of variations
        $variation_id = $this->mazzaneh_wcpe_find_matching_variation($product, $product->get_default_attributes());
        if ($variation_id != 0) {
          $variation = wc_get_product($variation_id);
          $temp_product->price = $variation->get_price();
          $temp_product->old_price = $variation->get_regular_price();
          $temp_product->availability = $variation->get_stock_status();
        } else {
          $temp_product->price = $product->get_variation_price('max');
          $temp_product->old_price = $product->get_variation_regular_price('max');
        }

        // Extract default attributes
        foreach ($product->get_default_attributes() as $key => $value) {
          if (!empty($value)) {
            if (substr($key, 0, 3) === 'pa_') {
              $value = get_term_by('slug', $value, $key);
              if ($value) {
                $value = $value->name;
              } else {
                $value = '';
              }
              $key = wc_attribute_label($key);
              $temp_product->attributes[urldecode($key)] = rawurldecode($value);
            } else {
              $temp_product->attributes[urldecode($key)] = rawurldecode($value);
            }
          }
        }
      }
      // add remain attributes
      foreach ($product->get_attributes() as $attribute) {
        if ($attribute['visible'] == 1) {
          $name = wc_attribute_label($attribute['name']);
          if (substr($attribute['name'], 0, 3) === 'pa_') {
            $values = wc_get_product_terms($product->get_id(), $attribute['name'], array('fields' => 'names'));
          } else {
            $values = $attribute['options'];
          }
          if (!array_key_exists($name, $temp_product->attributes)) {
            $temp_product->attributes[$name] = implode(', ', $values);
          }
        }
      }

      if (!array_key_exists('شناسه کالا', $temp_product->attributes)) {
        $sku = $product->get_sku();
        if ($sku != "") {
          $temp_product->attributes['شناسه کالا'] = $sku;
        }
      }

      if (count($temp_product->attributes) > 0) {
        $temp_product->attributes = [$temp_product->attributes];
      }

      return $temp_product;
    }

    /**
     * Find matching product and variation
     */
    private function mazzaneh_wcpe_find_matching_variation($product, $attributes)
    {
      foreach ($attributes as $key => $value) {
        if (strpos($key, 'attribute_') === 0) {
          continue;
        }
        unset($attributes[$key]);
        $attributes[sprintf('attribute_%s', $key)] = $value;
      }
      if (class_exists('WC_Data_Store')) {
        $data_store = WC_Data_Store::load('product');
        return $data_store->find_matching_product_variation($product, $attributes);
      } else {
        return $product->get_matching_variation($attributes);
      }
    }
  }
  $Mazzaneh_Woocommerce_Categories_Products_Extractor = new Mazzaneh_Woocommerce_Categories_Products_Extractor;
}
